c     Tells whether (ra,dec) is inside the area of region 'reg'. 
c     N.B.: Holes are not taken into account by this routine...
c
      subroutine inside_area(reg,ra,dec,in)

      implicit none
      
      integer in
      real ra,dec
      character reg*3
      
      integer infield(1),nf,sgn


      in=0
      if (reg.eq.'ran') then
         call inside_fields(reg,ra,dec,nf,infield)
         if (nf.eq.1) in=1
      else if (reg.eq.'sgp') then
         sgn=1
         call inside_plates(sgn,ra,dec,in)
      else if (reg.eq.'ngp') then
         sgn=-1
         call inside_plates(sgn,ra,dec,in)
      else
         stop 'inside_area: region not recognized!'
      endif

      return
      end

c-----------------------------------------------------------------------------
      subroutine inside_fields(reg,ra,dec,nf,infield)

      implicit none

      integer nf,infield(*)
      real ra,dec
      character reg*3

      integer NMAX
      real theta_min
      parameter (NMAX=700,theta_min=1.014)
      integer ifield(NMAX)
      real xc(NMAX),yc(NMAX),zc(NMAX)

      logical in
      integer i,nbf,ifirst
      real costheta_min,x,y,z,r,costheta_r,costheta
      character name*180,reglast*3
      data ifirst/1/
      save ifirst,reglast,costheta_min,nbf,ifield,xc,yc,zc
      

      if (ifirst.eq.1) then
         ifirst=0
         reglast='???'
         costheta_min=cos(theta_min*atan(1.)/45.)
      endif

      if (reg.ne.reglast) then
         reglast=reg
         name='/gal/dsk18/irpn/2df/old_format/data/'//reg//'centres.txt'
         call read_fields_xyz(name,reg,nbf,ifield,xc,yc,zc)
      endif


      call radec_xyz(ra,dec,x,y,z)
      r=sqrt(x**2+y**2+z**2)
      costheta_r=costheta_min*r

      i=0
      nf=0
      in=.false.
      do while ((.not.in).and.(i.lt.nbf))
         i=i+1
         costheta=(x*xc(i)+y*yc(i)+z*zc(i))
         if (costheta.gt.costheta_r) then 
            nf=nf+1  
            infield(nf)=ifield(i) 
            if (reg.eq.'ran') then ! We use here the fact the random
               in=.true.           ! fields doesn't overlapp with 
            else if (nf.gt.9) then ! each other... 
               in=.true.           ! No more than 8 2dF field can overlapp...
               write(0,*) 'Have I ever arrived here????'
            endif
         endif
      enddo

      return
      end

c-----------------------------------------------------------------------------
      subroutine inside_plates(sgn,ra,dec,in)
c     This subroutine tells whether a position in ra and dec is inside (in=1)
c     or not (in=0) the 2df_boundaries defined by the plates. 

      implicit none
      
      integer sgn,in
      real ra,dec

      integer jfld
      real  ra_r,pival,degs_rads,ramins_rads,radfac,tpi
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
      
      in=0      
      ra_r=ra
      
      call initialisation(sgn)

      if ((ra.gt.pival).and.(sgn.eq.1))then
         ra_r=ra-tpi
      endif
      call fnumber(ra_r,dec,jfld)

      if (sgn.eq.1) then
         if (jfld.lt.349 .or. jfld.gt.537) return
         if (jfld.gt.357 .and. jfld.lt.404) return
         if (jfld.gt.418 .and. jfld.lt.466) return
         if (jfld.gt.481 .and. jfld.lt.532) return
      else
         if (jfld.lt.781 .or. jfld.gt.867) return
         if (jfld.gt.795 .and. jfld.lt.853) return
      endif

      in=1

      return
      end

c-----------------------------------------------------------------------------
c This subroutine holes_2df tells if a pixel is in the sgp/ngp area or not, 
c depending on the value of in (in=1 inside; in=0 not inside). The value of 
c sgn tells if it is a position in the ngp or sgp and the pixel coordinate 
c is given by ix,iy. NP defines the general scale of the pixel map.

      subroutine holes_2df(NP,sgn,ix,iy,in)
*************************************variables********************************
      implicit none
      
      integer NP,sgn,ix,iy,in
     
      integer jf
      real rx,ry,ra,dec,x,y,z
      real pival,degs_rads,ramins_rads,radfac,tpi
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
******************************************************************************


      in=0      
      call initialisation(sgn)
      
      call ix_rx_map(NP,sgn,ix,iy,rx,ry)
      call inv_eq_2dfrx(NP,rx,ry,sgn,x,y,z)
      call xyz_radec(x,y,z,ra,dec)
      
      if ((ra.gt.pival).and.(sgn.eq.1))then
         ra=ra-tpi
      endif
      
      if (sgn.eq.1) then
         call test_sgp(ra,dec,in)
      else
         call test_ngp(ra,dec,in)
      endif
      
      return
      end

c-----------------------------------------------------------------------------
c This subroutine holes_2df_xyz tells if (x,y,z) is in the sgp/ngp area 
c or not, depending on the value of in (in=1 inside; in=0 not inside). 
      subroutine holes_2df_xyz(x,y,z,in)
*************************************variables********************************
      implicit none
      
      integer in
      real x,y,z

      integer jf,ifirst,sgn
      real ra,dec,racen,deccen,xc,yc,zc,PI,r,zz
      real pival,degs_rads,ramins_rads,radfac,tpi
      data ifirst/1/
      save ifirst,xc,yc,zc
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
******************************************************************************
      if (ifirst.eq.1) then
         ifirst=0
         PI= atan(1.)*4.
c        This is the direction of the Galactic Centre as adopted
c        by Steve Maddox
         racen  = 12.3*PI/180.0  
         deccen = -27.5*PI/180.0  
         xc=cos(deccen)*cos(racen)
         yc=cos(deccen)*sin(racen)
         zc=sin(deccen)
      endif

      in=0      
      r=sqrt(x**2+y**2+z**2)
      zz=(x*xc+y*yc+z*zc)
      sgn=int(sign(1.,zz/r))

      call initialisation(sgn)
      call xyz_radec(x,y,z,ra,dec)
      
      if ((ra.gt.pival).and.(sgn.eq.1))then
         ra=ra-tpi
      endif
      
      if (sgn.eq.1) then
         call test_sgp(ra,dec,in)
      else
         call test_ngp(ra,dec,in)
      endif
       
      return
      end

c-----------------------------------------------------------------------------
c This subroutine holes_2df_radec tells if (ra,dec) is in the sgp/ngp area 
c or not, depending on the value of in (in=1 inside; in=0 not inside).
      subroutine holes_2df_radec(ra,dec,in)
*************************************variables********************************
      implicit none
      
      integer in
      real ra,dec

      integer jf,ifirst,sgn
      real x,y,z,rar,racen,deccen,xc,yc,zc,PI,r,zz
      real pival,degs_rads,ramins_rads,radfac,tpi
      data ifirst/1/
      save ifirst,xc,yc,zc
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
******************************************************************************
      if (ifirst.eq.1) then
         ifirst=0
         PI= atan(1.)*4.
c        This is the direction of the Galactic Centre as adopted
c        by Steve Maddox
         racen  = 12.3*PI/180.0  
         deccen = -27.5*PI/180.0  
         xc=cos(deccen)*cos(racen)
         yc=cos(deccen)*sin(racen)
         zc=sin(deccen)
      endif

      in=0  
      call radec_xyz(ra,dec,x,y,z)
      r=sqrt(x**2+y**2+z**2)
      zz=(x*xc+y*yc+z*zc)
      sgn=int(sign(1.,zz/r))

      call initialisation(sgn)
 
      rar=ra
      if ((rar.gt.pival).and.(sgn.eq.1))then
         rar=rar-tpi
      endif
      
      if (sgn.eq.1) then
         call test_sgp(rar,dec,in)
      else
         call test_ngp(rar,dec,in)
      endif
      
      return
      end

c----------------------------------------------------------------------------
      subroutine initialisation(sgn)
     
      implicit none

      integer sgn

      integer sgn_ifirst
      save sgn_ifirst
      data sgn_ifirst/0/

      if (sgn.ne.sgn_ifirst) then
         sgn_ifirst=sgn
         if (sgn.eq.1) then
            call sgp_mask_init
         else
            call ngp_mask_init
         endif
      endif

      return
      end

c----------------------------------------------------------------------------
c$$$  The following subroutines has been sent by Helen Tadros in Oxford
c$$$  (March 2000), and has been written by several members of the 2dFGRS 
c$$$  team. The exact full code sent by Helen Tadros is in 
c$$$  /gal/dsk18/irpn/2df/holes_2df/mask_2df_gbd.f . 
c$$$  The version below is the upgraded and fully corrected version (by 
c$$$  your 2dF servant iprn). These routines are used by the fantastic 
c$$$  Durham mask software.
c---------------------------------------------------------------------------
      subroutine test_ngp(xx,yy,inngp)
c *** returns inngp = 1 if this object (xx,yy) in radians is in the NGP survey

      implicit none
      
      integer inngp
      real xx,yy

      integer jfld,inhol,ilast
      real pival,degs_rads,ramins_rads,radfac,tpi,x1,y1
      
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
      common/last_field/ilast,jfld


      inngp=0
      call fnumber(xx,yy,jfld)
      if (jfld.lt.781 .or. jfld.gt.867) return
      if (jfld.gt.795 .and. jfld.lt.853) return

      inhol = 0
      if (jfld.ne.ilast) call set_header(jfld)

      x1=xx
      y1=yy
      if (x1.lt.0) x1 = x1 + tpi
      call machin(x1,y1)
      !if ((xx.ge.3.089565).and.(xx.le.3.089567).and.(yy.le.-0.041629).and.(yy.ge.-0.041630)) write(0,*) 'x1,y1:',x1,y1
      call test_ngp_holes(x1,y1,jfld,inhol)

      if (inhol.eq.1) return
c     if we're still here we should be safe!
      inngp = 1
      return
      end

c----------------------------------------------------------------------------
      subroutine test_sgp(xx,yy,insgp)
c *** returns insgp = 1 if this object (x,y) in radians is in the SGP survey

      implicit none
      
      integer insgp
      real xx,yy

      integer jfld,inhol,ilast
      real pival,degs_rads,ramins_rads,radfac,tpi,x1,y1

      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
      common/last_field/ilast,jfld

      insgp=0
      call fnumber(xx,yy,jfld)
      if (jfld.lt.349 .or. jfld.gt.537) return
      if (jfld.gt.357 .and. jfld.lt.404) return
      if (jfld.gt.418 .and. jfld.lt.466) return
      if (jfld.gt.481 .and. jfld.lt.532) return

      inhol = 0
      if (jfld.ne.ilast) call set_header(jfld) 

      x1=xx
      y1=yy
      if (x1.gt.pival) x1 = x1 - tpi
      call machin(x1,y1)
      call test_sgp_holes(x1,y1,jfld,inhol)

      if (inhol.eq.1) return
c     if we're still here we should be safe!
      insgp = 1
      return
      end

c------------------------------------------------------------------------------
      subroutine test_hole(xx,yy,io)
c *** returns io = 0 if this object (xx,yy) in radians is in a drill hole
c     N.B.: Need to have called subroutine initialisation beforehand!

      implicit none
      
      integer io
      real xx,yy

      integer jfld,ilast,in
      real pival,degs_rads,ramins_rads,radfac,tpi,x1,y1
      !integer num
      !data num/0/
      !save num
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi
      common/last_field/ilast,jfld


      call fnumber(xx,yy,jfld)
      if ((jfld.eq.49).or.(jfld.eq.143).or.(jfld.eq.487)) then
c        These 3 UKST plates have been successfully checked: no data on them!
         io=0
         !num=num+1
         !write(0,*) 'Attempt to set header of non-existent fld : ',jfld,num
         return
      endif
      if (jfld.ne.ilast) call set_header(jfld)

      if (((jfld.ge.764).and.(jfld.le.815)).or.
     &    ((jfld.ge.835).and.(jfld.le.888))    ) then ! plate in the NGP region
         x1=xx
         y1=yy
         if (x1.lt.0) x1 = x1 + tpi 
         call machin(x1,y1)
         call test_ngp_holes(x1,y1,jfld,in)
      else                      ! plate in the SGP region
         x1=xx
         y1=yy
         if (x1.gt.pival) x1 = x1 - tpi
         call machin(x1,y1)
         call test_sgp_holes(x1,y1,jfld,in)
      endif

c     N.B.: in = 1 if in a hole and 0 otherwise (ie. opposite to io)
      io=abs(in-1) 
      
      return
      end

c---------------------------------------------------------------------------
      subroutine set_pivals 
c     Initialisation of some constants for these routines
      implicit none

      real pival,degs_rads,ramins_rads,radfac,tpi
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi

      pival       = atan(1.)*4.
      degs_rads   = pival/180.0
      ramins_rads = pival/(12.*60.) 
      radfac      = 2.*pival/360./60.
      tpi         = 2.*pival 
      return
      end

c-----------------------------------------------------------------------------
      subroutine fnumber(rar,decr,ifield)
c
c     given ra,dec in radians returns corresponding field no.
c     
      implicit none

      integer ifield
      real decr,rar

      integer iygrid,ixgrid,i
      real ra,dec,raminus,yspace,ygrid,xspace,xgrid,xoff,yoff
     &,    xspace348,ratest348,edge348

      integer*4 field0h(22)
      real*4 ragaps(22), decbands(22)

      data (ragaps(i),i=1,22)/ 0.,144.,90.,66.,52.,44.,38.,33.,30.,28.,
     +                 26.,24.,23.,22.,21.,20.,20.,20.,20.,20.,20.,20./
      data (decbands(i),i=1,22)/-90.,-85.,-80.,-75.,-70.,-65.,-60.,-55.,
     +   -50.,-45.,-40.,-35.,-30.,-25.,-20.,-15.,-10.,-5.,0.,5.,10.,15./
      data (field0h(i),i=1,22)/1,2,12,28,50,78,111,149,193,241,293,349,
     +   409,472,538,607,679,751,823,895,967,1039 / 

      real pival,degs_rads,ramins_rads,radfac,tpi
      common/pivals/ pival,degs_rads,ramins_rads,radfac,tpi


c     find the nearest grid centre

      ra = rar*180./pival
      dec = decr*180./pival
      
      if (dec.gt.15.) then
         ifield = 0
         return
      end if
      
      if (ra.lt.0.) ra = ra + 360.
      if (ra.gt.180.) then
         raminus = ra - 360.
      else
         raminus = ra
      end if
      
      yspace = 5.0
      iygrid = nint(dec/5.)+19
      ygrid = decbands(iygrid)
      
      xspace = ragaps(iygrid)
      if (abs(raminus).lt.xspace/4.0/2.0) then ! 4 for degs, 2 for half field
c     it's a 0hr field
         xgrid = 0.
         ixgrid = 0
      else 
         xspace = ragaps(iygrid) / 4.0 ! in degs
         ixgrid = int(ra/xspace + 0.5)
         xgrid = ixgrid * xspace
      end if
      
      ifield = field0h(iygrid) + ixgrid

c     Do we really need this part???? Apparently for the SGP region...
      xspace348   = 26
      ratest348   = 0.5*xspace348*ramins_rads
      edge348     = -10.*ramins_rads +ratest348
      if(ifield.eq.293 .and. raminus.le.edge348) then
         ifield=348
         xgrid = -10*xspace348
      end if
c     ?????

      xoff = ra - xgrid
      if (xoff.gt.180.) xoff = xoff - 360.
      yoff = dec - ygrid
c     type*,'Field no. = ',ifield
c     type*,'RA , Dec offset = ',xoff,yoff
      
      return
      end
c-----------------------------------------------------------------------------
      subroutine sgp_mask_init
c     Initialisation of the sgp_holes positions

      implicit none

      integer i,j,jhole(2000),nshols,ilast,jfld
      real sxhole(3,2000),syhole(3,2000)

      common/sgphole/sxhole,syhole,jhole,nshols
      common/last_field/ilast,jfld


      call set_pivals
      call read_dist
      call read_headers
      ilast = 0
      open(unit=30,file='sub/sgpholes.lis',status='old',form='formatted')
      do i =1,2000
         read(30,*,end=999) (sxhole(j,i),syhole(j,i),j=1,3),jhole(i)
      end do
 999  nshols = i-1
      close(30)
      write(0,*) nshols,' SGP holes read'
      return
      end
c-----------------------------------------------------------------------------
      subroutine ngp_mask_init
c     Initialisation of the ngp_holes positions
c     These holes are now in X,Y on each plate.
      implicit none

      integer i,j,jhole(300),nnhols,ilast,jfld
      real xnhole(3,300),ynhole(3,300)

      common/ngphole/xnhole,ynhole,jhole,nnhols
      common/last_field/ilast,jfld

      call set_pivals
      call read_dist
      call read_headers
      ilast=0 ! modification by iprn (17.04.2002)    
      open(unit=30,file='sub/ngpholes.lis',status='old',form='formatted')
      do i =1,300
         read(30,*,end=999) (xnhole(j,i),ynhole(j,i),j=1,3),jhole(i)
      end do
 999  nnhols = i-1
      close(30)
      write(0,*) nnhols,' NGP holes read'
      return
      end
c-----------------------------------------------------------------------------
      subroutine test_sgp_holes(xx,yy,jf,inhol)
c     
c     Returns inhol = 1 if point (xx,yy) lies in a drilled region
c
c     Express displacement vector to (xx,yy) in terms of basis vectors of
c     paralellogram
c
c       Vr = a1*V1 + a2*V2
c       -       -       -
c
c     Then  drilled if  0 <= a1,a2 <= 1
c     Altered by WJS from JL's DRILLED routine, 4/10/89

** Note: point 2 is sometimes bottom & sometimes top of hole,
**  but 1 and 3 always seem to be left & right hand sides.
*
*  ie    1 3   or  2
*        2         1 3

      implicit none

      integer jf,inhol
      real xx,yy

      integer j,jhole(2000),nholes
      real xhole(3,2000),yhole(3,2000),dx,dy,dx1,dy1,dx2,dy2,x0,y0,a1,a2

      common/sgphole/xhole,yhole,jhole,nholes


      inhol = 0
      do j = 1,nholes
         if (jhole(j).ne.jf) goto 124
         x0 = xhole(1,j)
         y0 = yhole(1,j)
         
         if(y0.eq.yhole(3,j)) then ! It's a square hole
            if(xx.le.x0.or.xx.ge.xhole(3,j)) goto 124 ! speed-up 
         end if
         dx = xx-x0
         dy = yy-y0
         dx1 = xhole(2,j)-x0
         dy1 = yhole(2,j)-y0
         dx2 = xhole(3,j)-x0
         dy2 = yhole(3,j)-y0
     
         if (abs(dx2).gt.0) then
            a1 = (dx*dy2 - dy*dx2)/(dx1*dy2 - dy1*dx2)
            a2 = (dx - a1*dx1)/dx2
            if (0.le.a1.and.a1.le.1.and.0.le.a2.and.a2.le.1) then
               inhol = 1
               return
            end if
         end if 
 124     continue
      end do                    ! loop over holes
      
      end
c-----------------------------------------------------------------------------
      subroutine test_ngp_holes(xx,yy,jf,inhol)
c     
c     Returns inhol = 1 if point (xx,yy) lies in a drilled region
c
c     Express displacement vector to (xx,yy) in terms of basis vectors of
c     paralellogram
c
c       Vr = a1*V1 + a2*V2
c       -       -       -
c
c     Then  drilled if  0 <= a1,a2 <= 1
c     Altered by WJS from JL's DRILLED routine, 4/10/89

** Note: point 2 is sometimes bottom & sometimes top of hole,
**  but 1 and 3 always seem to be left & right hand sides.
*
*  ie    1 3   or  2
*        2         1 3

      implicit none

      integer jf,inhol
      real xx,yy

      integer j,jhole(300),nholes
      real xhole(3,300),yhole(3,300),dx,dy,dx1,dy1,dx2,dy2,x0,y0,a1,a2

      common/ngphole/xhole,yhole,jhole,nholes


      inhol = 0
      do j = 1,nholes
         if (jhole(j).ne.jf) goto 123
         x0 = xhole(1,j)
         y0 = yhole(1,j)
         if(y0.eq.yhole(3,j)) then ! It's a square hole
            if(xx.le.x0.or.xx.ge.xhole(3,j)) goto 123 ! speed-up 
         end if
         dx = xx-x0
         dy = yy-y0
         dx1 = xhole(2,j)-x0
         dy1 = yhole(2,j)-y0
         dx2 = xhole(3,j)-x0
         dy2 = yhole(3,j)-y0
     
         if (abs(dx2).gt.0) then
            a1 = (dx*dy2 - dy*dx2)/(dx1*dy2 - dy1*dx2)
            a2 = (dx - a1*dx1)/dx2
            if (0.le.a1.and.a1.le.1.and.0.le.a2.and.a2.le.1) then
               inhol = 1
               return
            end if
         end if 
 123     continue
      end do  ! loop over holes

      end

c----------------------------------------------------------------------------
      subroutine machin(x,y)
c     MACHIN:  converts ra and dec to machine coordinates
c     This version includes distortion corrections
      implicit none

      real x,y

      real ra,dec,zz,xi,xn,rr,denom
      real  xscale,yscale,xstart,ystart,a,b,c,d,e,f,tand,secd,tpa,tpd
     &,     xscale1,yscale1

      common /rd/ xscale,yscale,xstart,ystart,a,b,c,d,e,f,tand,
     &            secd,tpa,tpd
      common /flip/xscale1,yscale1

      
      ra=x
      dec=y
      zz = 1./(tand*tan(dec)+cos(ra-tpa))
      xi=secd*sin(ra-tpa)*zz
      xn=(tan(dec)-tand*cos(ra-tpa))*zz
      rr=sqrt(xi**2+xn**2)
      if(rr.eq.0.0)then
        rr=1.0
      else
        rr=rr/tan(rr)
      endif
      xi=xi*rr-c
      xn=xn*rr-f
      call corr_dist(xi,xn,0) 
      denom=(a*e-d*b)
      x=(xi*e-xn*b)/denom
      y=(xn*a-xi*d)/denom
      x=(x-xstart)*xscale1
      y=(y-ystart)*yscale1
      return
      end

c-----------------------------------------------------------------------------
      subroutine read_dist 
c     Read distortion file
      implicit none

      integer ibins,n,io20,ixi,ixn,nbins
      real xi,xn,ddxi,ddxn,ximin,xnmin,scale,radeg,conver
      integer i,ixifil,ixnfil,ixi1,ixn1,i2
      real*4 dxi(30,30), dxn(30,30)
      real*4 dxim(30,30), dxnm(30,30)
      real*4 dxihist(9), dxnhist(9)

      common/distortion/ dxi, dxn, ximin, xnmin, scale, n 

c$$$c     This part needs to be added if one uses the dodgy bit...
c$$$      integer ifirst
c$$$      data ifirst/1/
c$$$      save ifirst

c$$$      if (ifirst.eq.1) then
c$$$         ifirst=0
c$$$      else
c$$$         return
c$$$      endif

      radeg=45.0/atan(1.0)
      conver=radeg*3600.0
      ibins = 0
      open(unit=30,file='sub/distortion',form='formatted',status='old'
     &,    access='sequential') 
      read(30,*,iostat=io20) ximin, xnmin, scale, n 
      nbins = n*n
      io20 = 0 
      do while(io20.eq.0) 
        read(30,*,iostat=io20) xi, xn, ddxi, ddxn 
        if(io20.eq.0) then 
          ibins = ibins + 1 
          ixi = min(n,max(1,int((xi-ximin)*scale)+1 ))
          ixn = min(n,max(1,int((xn-xnmin)*scale)+1 ))
          dxi(ixi,ixn) = ddxi/conver
          dxn(ixi,ixn) = ddxn/conver
        end if
      end do 
      close(30)


c     Do we really need this part: it has been added since a comparison 
c     was done between this code and Steve Maddox's code, who apprently 
c     uses this part (for some unknown reason: I just hope this is not 
c     unknown to himself also!)
c     N.B.: This part is subject to the number of times the routine 
c     read_dist is called! So what is the trust we place in it????
c$$$*       median filter
c$$$      do ixn = 1,n
c$$$         do ixi = 1,n
c$$$            i = 0
c$$$            do ixnfil = -1,1
c$$$               ixn1 = ixn+ixnfil
c$$$               if(ixn1.ge.1 .and. ixn1.le.n) then
c$$$                  do ixifil = -1,1
c$$$                     ixi1 = ixi+ixifil
c$$$                     if(ixi1.ge.1 .and. ixi1.le.n) then
c$$$                        i = i+1
c$$$                        dxihist(i) = dxi(ixi1,ixn1)
c$$$                        dxnhist(i) = dxn(ixi1,ixn1)
c$$$                     end if
c$$$                  end do
c$$$               end if
c$$$            end do
c$$$            
c$$$            call sort1(dxihist,i)
c$$$            call sort1(dxnhist,i)
c$$$            
c$$$            i2 = i/2
c$$$            if(i2*2.eq.i) then
c$$$               dxim(ixi,ixn) = (dxihist(i2)+dxihist(i2+1) )/2.
c$$$               dxnm(ixi,ixn) = (dxnhist(i2)+dxnhist(i2+1) )/2
c$$$            else
c$$$               dxim(ixi,ixn) = dxihist(i2+1)
c$$$               dxnm(ixi,ixn) = dxnhist(i2+1)
c$$$            end if
c$$$            
c$$$         end do
c$$$      end do
c$$$      
c$$$      do ixn = 1,n
c$$$         do ixi = 1,n
c$$$            dxi(ixi,ixn) = dxim(ixi,ixn)
c$$$            dxn(ixi,ixn) = dxnm(ixi,ixn)
c$$$         end do
c$$$      end do
c     End of added bit!

      if (nbins.ne.ibins) 
     &   write(0,*) ibins,' out of',nbins,' distortion bins filled'
      return
      end

c------------------------------------------------------------------------
      subroutine sort1(ia,n) ! a well documented routine provided by ...
      !real ia(n),it
      implicit none

      integer n
      real ia(n)

      integer int,ifin,ii,i,j
      real it


      int=2
   10 int=2*int
      if(int.lt.n)goto 10
      int=min0(n,(3*int)/4-1)
   20 int=int/2
      ifin=n-int
      do 70 ii=1,ifin
      i=ii
      j=i+int
      if(ia(i).le.ia(j))goto 70
      it=ia(j)
   40 ia(j)=ia(i)
      j=i
      i=i-int
      if(i.le.0)goto 60
      if(ia(i).gt.it)goto 40
   60 ia(j)=it
   70 continue
      if(int.gt.1)goto 20
      return
      end

c------------------------------------------------------------------------
      subroutine corr_dist(xi,xn,iforw) 

c     iforw determines which way the Schmidt distortion map is applied: 
c     either to go forward from measured (x,y) to get (ra,dec) or backwards 
c     from (ra,dec) to (x,y). It should be set to 1 when called from radecd
c     and 0 when called from machin.
 
      implicit none

      integer iforw
      real*4 xi, xn

      integer n,ixi1,ixi2,ixn1,ixn2,ilast,jfld
      real  ximin,xnmin,scale,axi,axn,rxi1,rxi2,rxn1,rxn2,area,dxia,dxib
     &,     dxic,dxid,corr_xi,dxna,dxnb,dxnc,dxnd,corr_xn
      real*4 dxi(30,30), dxn(30,30) 

      common/last_field/ilast,jfld
      common/distortion/ dxi, dxn, ximin, xnmin, scale, n 


      axi = (xi-ximin)*scale
      axn = (xn-xnmin)*scale
      ixi1 = min(n-1,max(1,int(axi)+1 ))
      ixi2 = min(n,max(2, ixi1 +1 )) 
      ixn1 = min(n-1,max(1,int(axn)+1 ))
      ixn2 = min(n,max(2, ixn1 +1 )) 

      rxi1 = axi - float(ixi1) 
      rxi2 = float(ixi2) - axi 

      rxn1 = axn - float(ixn1) 
      rxn2 = float(ixn2) - axn 

      area = (rxi1+rxi2)*(rxn1+rxn2)
      if (area.eq.0.) then
         write(0,*) rxi1,rxi2,rxn1,rxn2
         !write(0,*) ilast,jfld
         stop 'holes.f: Area=0 in corr_dist !'
      end if
      dxia = dxi(ixi1,ixn1) * rxi2 * rxn2 
      dxib = dxi(ixi2,ixn1) * rxi1 * rxn2 
      dxic = dxi(ixi2,ixn2) * rxi1 * rxn1 
      dxid = dxi(ixi1,ixn2) * rxi2 * rxn1 
      corr_xi = (dxia+dxib+dxic+dxid) / area

      dxna = dxn(ixi1,ixn1) * rxi2 * rxn2 
      dxnb = dxn(ixi2,ixn1) * rxi1 * rxn2 
      dxnc = dxn(ixi2,ixn2) * rxi1 * rxn1 
      dxnd = dxn(ixi1,ixn2) * rxi2 * rxn1 
      corr_xn = (dxna+dxnb+dxnc+dxnd) / area

      if(iforw.eq.1) then 
        xi = xi + corr_xi
        xn = xn + corr_xn
      else 
        xi = xi - corr_xi
        xn = xn - corr_xn
      end if

      return 
      end

c-----------------------------------------------------------------------------
      subroutine read_headers
c     iharray contains the fld numbers and harray contains the corresponding 
c     header buffers
      
      implicit none

      integer i,j

      real*4 harray(128,500)
      integer*2 iharray(500)
      common/all_headers/harray,iharray


      open(unit=30,file='sub/allheaders.dat',status='old',form='unformatted')
      read(30) (iharray(i),i=1,500)
      do i=1,500
         read(30) (harray(j,i),j=1,128)
      enddo
      close(30)
c      write(0,*) 'Allheaders read'

      return
      end

c----------------------------------------------------------------------------
      subroutine set_header(jfld)

      implicit none

      integer jfld,jfld1
      integer ilast,ii,i

      real*4 harray(128,500),buf(128)
      integer*2 iharray(500)
      common/all_headers/harray,iharray
      common/last_field/ilast,jfld1


      if (ilast.eq.jfld) return
      ii=1 ! modification by iprn (03.03.2000): previously it was: ii = 0
      do while(iharray(ii).ne.jfld .and. ii.lt.500)
         ii=ii+1
      end do
      if (ii.eq.500) then
         write(0,*) 'WARNING: Attempt to set header of non-existent fld',jfld
c         return  ! modification by iprn (03.03.2000)
         stop 'holes.f: too big value for header in set_header!'
      end if
      do i = 1,128
         buf(i) = harray(i,ii)
      end do

c     Set coord transforms for field jfld
      call rdset(buf,jfld)
      ilast = jfld
      jfld1=jfld ! modification by iprn (17.04.2002)
      return
      end

c-----------------------------------------------------------------------------
c$$$c N.B.: This version, provided by Steve Maddox, works perfectly fine...
c$$$c       that's why we have exchanged it for another routine...
c$$$c 
c$$$      subroutine rdset(tbuf,jfld)
c$$$
c$$$      implicit none
c$$$      integer jfld
c$$$      real*4 tbuf(128)
c$$$
c$$$      integer ima
c$$$      real  xscale,yscale,xstart,ystart,a,b,c,d,e,f,tand,secd,tpa,tpd
c$$$     &,     xscale1,yscale1!,xscaleb,yscaleb,xstartb,ystartb,xcen,ycen
c$$$      common /rd/ xscale,yscale,xstart,ystart,a,b,c,d,e,f,tand
c$$$     &,           secd,tpa,tpd
c$$$      !common /bmp/xscaleb,yscaleb,xstartb,ystartb,xcen,ycen
c$$$      common /flip/xscale1,yscale1
c$$$
c$$$
c$$$      a = tbuf(65)
c$$$      b = tbuf(66)
c$$$      c = tbuf(67)
c$$$      d = tbuf(68)
c$$$      e = tbuf(69)
c$$$      f = tbuf(70)
c$$$      tpa = tbuf(71)
c$$$      tpd = tbuf(72)
c$$$      tand = tan(tpd)
c$$$      secd = 1.0/cos(tpd)
c$$$c
c$$$      ima=0
c$$$      if(tbuf(127).eq.-1.0.or.tbuf(127).eq.-2.0)ima=1
c$$$      if(abs(tbuf(73)-0.00076).lt.0.0001.and.tbuf(126).ne.-1.0.
c$$$     &   and.ima.eq.0)then
c$$$        xscale = 0.0008
c$$$        yscale = -0.0008
c$$$      else 
c$$$        xscale = tbuf(73)
c$$$        yscale = tbuf(74)
c$$$      end if
c$$$
c$$$      xstart = 0.0
c$$$      ystart = 0.0
c$$$c
c$$$      !xscaleb = tbuf(73)
c$$$      !yscaleb = tbuf(74)
c$$$      !xstartb = tbuf(75)
c$$$      !ystartb = tbuf(76)
c$$$c
c$$$      xscale1 = 1./xscale !added by IPRN
c$$$      yscale1 = 1./yscale !added by IPRN
c$$$
c$$$      return
c$$$      end
c$$$
c----------------------------------------------------------------------------
      subroutine rdset(tbuf,jfld)

      implicit none

      integer jfld
      real*4 tbuf(128)

      real  xscale,yscale,xstart,ystart,a,b,c,d,e,f,tand,secd,tpa,tpd
     &,     xscaleb,yscaleb,xstartb,ystartb,xcen,ycen,xscale1,yscale1
      common /rd/ xscale,yscale,xstart,ystart,a,b,c,d,e,f,tand
     &,           secd,tpa,tpd
      common /bmp/xscaleb,yscaleb,xstartb,ystartb,xcen,ycen
      common /flip/xscale1,yscale1


      a = tbuf(65)
      b = tbuf(66)
      c = tbuf(67)
      d = tbuf(68)
      e = tbuf(69)
      f = tbuf(70)
      tpa = tbuf(71)
      tpd = tbuf(72)
      tand = tan(tpd)
      secd = 1.0/cos(tpd)

      xscale = 0.0008
      yscale = -0.0008

c     UKST Plates which have been rescaled:
c     414  417 481 782 783 784 785 786 792 793 794 795 
c     853 854 855 857 858 862 865
c     But what about: 471, 479 & 535 ? They have also been rescalled 
c     according to the latest 2dFGRS gossip... 
c     (see Maddox's e-mail sent on Friday 26.04.2002)

      if( jfld.eq.414 .or. jfld.eq.417 .or. jfld.eq.471 .or. jfld.eq.479 
     &    .or. jfld.eq.481 .or. jfld.eq.535  .or. jfld.eq.857  
     &    .or. (jfld.ge.782 .and. jfld.le.786) .or. jfld.eq.858
     &    .or. (jfld.ge.792 .and. jfld.le.795) .or. jfld.eq.862
     &    .or. (jfld.ge.853 .and. jfld.le.855) .or. jfld.eq.865 ) then 
 
         xscaleb = tbuf(73)
         yscaleb = tbuf(74)
         xstartb = tbuf(75)
         ystartb = tbuf(76)
         
         xscale = xscaleb
         yscale = yscaleb

      endif

      xscale1 = 1./xscale
      yscale1 = 1./yscale
      xstart = 0.0
      ystart = 0.0


      return
      end




