c-----------------------------------------------------------------------------
c     
c     COMPILATION OF THE CODES: ranmag.f, ran_subs.f, geometries.f 
c
c-----------------------------------------------------------------------------
c
c     RANMAG.F (essentials from...)
c
c-----------------------------------------------------------------------------
c Assign a random magnitude consistent with the selection function
c
c
c    
      real function ranmag_z(zplus1,blim,iseed)
*************************************variables********************************
      implicit none
      integer iseed
      real EPS,xp,dx,tacc,acc,t,dlzc,r,z,alpha,selx,f,ran3,dsel,selt
     &,    bstarz,x,rtsafe,zplus1,dl,bstar1,alpha1
      parameter(dx=1.0,acc=0.009,EPS=0.01,alpha1=0.0,bstar1=0.)
      real epsf
      real alpha0,bstar0,psi0,blim
      common /selection_ranmag/ alpha0,bstar0,psi0
      common /seld_par/ alpha,selx,f  
      external seld
******************************************************************************

      z = zplus1-1.0                        !redshift z
      call search_dist(r,zplus1,dl,dlzc,2)  !find corresponding dl

      bstarz = bstar0+bstar1*z
c     x=L_limit/L_star
      x = dlzc * 10**(10.0 -0.4*(blim-bstarz))
      alpha = alpha0+alpha1*z
      f = 0.0
      selx = 1.0
      call seld(x,selx,dsel)  !selection function at blim
      if (selx.le.0.0) then
         write(0,*) 'x=',x
         write(0,*) 'z=',z
         write(0,*) 'dlzc=',dlzc
      end if
c
c     Find Root that determines t=L/L_star
      f=ran3(iseed)           !random number [0,1]
      if (f.lt.1.0) then      
         xp=x+dx                !bracket root of selt/selx - f = 0 
         epsf=-EPS*f            !small offset to ensure root is bracketed
         call seld(xp,selt,dsel) 
         do while (selt.ge.epsf) 
            xp=xp+dx
            call seld(xp,selt,dsel)
         end do
         tacc=x*acc             !magnitude error < acc*2.5/ln(10)
         t=rtsafe(seld,x,xp,tacc)
      else   !this can happen with lots of randoms and
         t=x !then causes the root finder to fail so instea
      end if !we catch it here
c
      ranmag_z=bstarz+2.5*(10.0-log10(t/dlzc))

      return
      end

c-----------------------------------------------------------------------------
c Assign a random apparent magnitude consistent with the selection function
c and also compute the maximum luminosity distance at which this galaxy 
c would still be included in the catalog taking account of the k-correction
c and how the galaxy would have evolved.
c    
      real function ranmag(r,blim,zmax,iseed)
*************************************variables********************************
      implicit none
      integer iseed
      real EPS,xp,dx,tacc,acc,t,dlzc,zmax,rmax,dmax
     &,    r,z,alpha,selx,f,ran3,dsel,selt,bstarz,x,rtsafe,zplus1,dl
     &,    bstar1,alpha1
      parameter(dx=1.0,acc=0.009,EPS=0.01,alpha1=0.0,bstar1=0.)
      real epsf
      real alpha0,bstar0,psi0,blim
      common /selection_ranmag/ alpha0,bstar0,psi0
      common /seld_par/ alpha,selx,f  
      external seld
******************************************************************************

      call search_dist(r,zplus1,dl,dlzc,1)  !find corresponding zplus1 and dl
      z = zplus1-1.0                        !and redshift z
      bstarz = bstar0+bstar1*z
c  x=L_limit/L_star
      x = dlzc * 10**(10.0 -0.4*(blim-bstarz))
      alpha = alpha0+alpha1*z
      f = 0.0
      selx = 1.0
      call seld(x,selx,dsel)  !selection function at blim
c
c  Find Root that determines t=L/L_star
      f=ran3(iseed)           !random number [0,1]
      xp=x+dx
      call seld(xp,selt,dsel) !bracket root of selt/selx - f = 0 
      epsf=-EPS*f             !small offset to ensure root is bracketed
      do while (selt.ge.epsf) 
         xp=xp+dx
         call seld(xp,selt,dsel)
      end do
      tacc=x*acc  !magnitude error < acc*2.5/ln(10)
      t=rtsafe(seld,x,xp,tacc)
c
      ranmag=bstarz+2.5*(10.0-log10(t/dlzc))
c     compute dl^2/(1+z)^beta at distance for which the apparent magnitude
c     of this galaxy would equal the survey magnitude limit. The beta
c     dependence in this is the "k+e-correction".
      dlzc=dlzc * 10.0**(0.4*(blim-ranmag))
c     find corresponding zplus1 & dl
      call search_dist(rmax,zplus1,dmax,dlzc,4)
      zmax=zplus1-1.0
      return
      end

c-----------------------------------------------------------------------------
c Returns scaled selection function and its derivative
c 
c     
      subroutine  seld(t,sel,dsel)
*************************************variables********************************
      implicit none
      real t,sel,dsel,alpha,selx,f,expint
      common /seld_par/ alpha,selx,f        
******************************************************************************
      sel=expint(t,alpha)/selx-f
      dsel= -((t**(-alpha))*exp(-t))/selx
      return
      end

c-----------------------------------------------------------------------------
c Evaluates the integal with accuracy exceeding 1 part in 1000
c
c int_x^infty t^-alpha exp(-t) dt
c 
c This integral which crops up when integrating the Schechter function
c is related to the incomplete Gamma function. This subroutine borrows
c some code from the Numerical Recipes routines gcf() 
c     
      real function  expint(x,alpha)
************************************variables*********************************
      implicit none
      integer ITMAX,n
      real x,alpha,EPS,g,gold,a0,a1,b0,b1,fac,an,ana,a,anf,XS,xx
      PARAMETER (ITMAX=1000,EPS=3.E-7,XS=0.1)
******************************************************************************
      a=1.0-alpha
      xx=max(x,XS)         
      gold=0.0
      a0=1.0
      a1=xx
      b0=0.0
      b1=1.0
      fac=1.0
      do   n=1,ITMAX
         an=float(n)
         ana=an-a
         a0=(a1+a0*ana)*fac
         b0=(b1+b0*ana)*fac
         anf=an*fac
         a1=xx*a0+anf*a1
         b1=xx*b0+anf*b1
         if(a1.ne.0.0)then
            fac=1.0/a1
            g=b1*fac
            if (abs((g-gold)/g).lt.EPS) goto 2
            gold=g
         endif
      end do
      stop 'a too large, itmax too small'
 2    expint = g*exp(-xx+a*alog(xx))
c     
c     For low values of x approximate exp(-x) as 1-x for x<XS
c     if alpha>1 ie a<0
      if (x.lt.XS) then
         a1=1.0+a
         expint=expint +(XS**a-x**a)/a + (x**a1-XS**a1)/a1
      end if
      return
      end

c-----------------------------------------------------------------------------
c
c     RAN_SUBS_SHAUN.f (essentials from...)
c
c-----------------------------------------------------------------------------
c Generates Random Galaxy positions. Improved version with a preliminary
c selection of the region where to put the randoms if one uses 2dF data.
c For the others surveys, put random galaxy position over the whole celestial
c sphere.
c
c  Input:
c     iseed   = random number seed (should be a negative integer)
c     zlimit  = redshift at which to truncate the catalogue eg z=0.5
c     selfile = name of file containing the tabulated selection function
c               eg om1:lam0:strc:a/sel1.2df_sgp
c                  (or 00002df_sgp for a volume limited catalogue)
c     vlim    = set .TRUE. if a volume limited cat is required
c
c     vran    = 1-dimensional random velocity dispersion in units
c               of km/s (set in parameter statement)
c
c  Output:
c     x,y,z  = Galaxy redshift space position
c     d      = Galaxy redshift if on pure Hubble Flow. NB Will equal
c               sqrt(x^2+y^2+z^2) if vran = 0.0 km/s .
c
c  N.B.: reg is declared as a character*3, which means that sdss & pscz
c        are written like dss & scz...
c
      subroutine ran_cat(x,y,z,d,iseed,zlimit,vlim,init,reg)

      implicit none

      integer iseed,ifirst,nbf,nmax
      character reg*3
      real x,y,z,d,vx,vy,vz,zlimit,z_gal,z_hub,v_dot_rhat,vran,invd,PI
      parameter (vran=0.0,PI=3.141592654,nmax=100)
      real xc(nmax),yc(nmax),zc(nmax),rac(nmax),decc(nmax)
      logical in,vlim,init
      data ifirst/1/
      real ran3,gasdev,dcos,costhemin,dphi,phimin,costheta,phi,sintheta
      save dcos,costhemin,dphi,phimin,ifirst
      real area
      integer ntries,nb_field
      common /area_tries/ area,ntries

c     Identify whether it is "dss", "apm" or 2dF "sgp" or "ngp" or
c     "ran" so that the appropriate geometrical boundaries can be applied.

      if ((ifirst.eq.1).or.(init)) then
         init=.false.
         ifirst=0
         if      (reg.eq.'ngp') then
            call geometry_2df_ngp2(dcos,costhemin,dphi,phimin)
         else if (reg.eq.'sgp') then
            call geometry_2df_sgp2(dcos,costhemin,dphi,phimin)
         else if (reg.eq.'ran') then ! can be improved...
            call geometry_2df_ran2(dcos,costhemin,dphi,phimin,nbf)
         else
c        Can be improved for sdss, pscz & .apm cases: now the whole sphere 
c        is used!
            dcos=2.
            costhemin=-1.
            dphi=8.*atan(1.)
            phimin=0.
         endif
c        record basic solid angle over which randoms are generated
c        Note special case for reg.eq.ran needed due to call to
c        geometry_2df_ran3() 
         area= dcos*dphi
         if (reg.eq.'ran') then
            call ran2df_cen(nb_field,xc,yc,zc,rac,decc)
            area=real(nb_field)*PI*(1.014*PI/180.0)**2
         end if
       endif

c     We now loop until we find a galaxy in the catalogue
      in=.FALSE.
      ntries=0   
      do while (.not.in)
c        Generate a random real space position on a portion of sphere of unit 
c        radius. The portion being defined by dphi,phimin & dcos,costhemin.
c        Using standard spherical coordinates
         if (reg.eq.'ran') 
     &      call geometry_2df_ran3(iseed,dcos,costhemin,dphi,phimin,nbf)
         costheta=dcos*ran3(iseed)+costhemin
         phi=dphi*ran3(iseed)+phimin
         sintheta=sqrt(1.-costheta**2) !quicker than sin(acos(costheta))
         x=cos(phi)*sintheta
         y=sin(phi)*sintheta
         z=costheta
         d=1.
         
         if      (reg.eq.'sgp') then
            call geometry_2df_sgp(in,x,y,z,d)
         else if (reg.eq.'ngp') then
            call geometry_2df_ngp(in,x,y,z,d)
         else if (reg.eq.'ran') then ! 2df random fields
            call geometry_2df_ran(in,x,y,z,d,nbf)
         end if

         ntries=ntries+1 !count the number of tries taken
         if (in) then       
c           Generate redshift randomly from the tabulated N(<z).
            call get_redshift_ran(z_hub,iseed)
            invd=1.0/d
     
c           Select a random velocity. 
            if (vran.gt.0.0) then
c              Each component chosen from a Gaussian of sigma=vran km/s 
               vx=gasdev(iseed)*vran
               vy=gasdev(iseed)*vran
               vz=gasdev(iseed)*vran
c              Compute velocity Component along the line of sight
               v_dot_rhat=(vx*x + vy*y + vz*z)*invd
c              and compute redshift via  (1+z_obs)=(1+z_hub)(1+v/c)
               z_gal   = ((1.0+z_hub)*(1+v_dot_rhat/3.0E+05)) - 1.0
               x = x*z_gal*invd !scale to redshift-space positions
               y = y*z_gal*invd
               z = z*z_gal*invd
            else
               x = x*z_hub*invd !scale to redshift-space positions
               y = y*z_hub*invd
               z = z*z_hub*invd
            endif
            d = z_hub           !save redshift if on Hubble Flow   
         endif
         if (reg.eq.'ran') ntries=1  !special case as area assumes always in
      enddo

      return
      end

c---------------------------------------------------
c  Generate a redshift consistent with the tabulated distribution
c
c  Method:
c    1) Generate a random number, nran, in the interval [0,N(z<zlimit)].
c    2) Search the table to find the redshift at which nran=N(<z) and
c    return z
c
      subroutine get_redshift_ran(z_hub,iseed)
      implicit none
      integer nrmax,ir,iseed,nr
      parameter (nrmax=2000)
      real nltz(nrmax),z(nrmax),sel(nrmax),nran,ran3,h,z_hub
      common/selfun_ran/ nltz,z,sel,nr
c
c     Generate random value in the interval [0,N(z<zlimit)]
      nran=ran3(iseed)*nltz(nr)
c     Locate the closest redshift
      call locate(nltz,nr,nran,ir)  !numerical recipes routine
      ir=max(ir,1)
c     Interpolate
      h=(nran-nltz(ir))/(nltz(ir+1)-nltz(ir))
      z_hub=z(ir+1)*h+z(ir)*(1.0-h)
      return
      end
c --------------------------------------------------
c  Look up the selection function at the specified redshift
c
c
      subroutine get_sel(z_hub,selz)
      implicit none
      integer nrmax,ir,nr
      parameter (nrmax=2000)
      real nltz(nrmax),z(nrmax),sel(nrmax),selz,h,z_hub
      common/selfun/ nltz,z,sel,nr
c     Locate the closest redshift
      call locate(z,nr,z_hub,ir)  !numerical recipes routine
      ir=max(ir,1)
c     Interpolate
      h=(z_hub-z(ir))/(z(ir+1)-z(ir))
      selz=sel(ir+1)*h+sel(ir)*(1.0-h)
      return
      end
c ------------------------------------------------------------
c  Look up the selection function at the specified redshift
c  from that tabulated in common block /selfun_ran/
c
      subroutine get_sel_ran(z_hub,selz)
      implicit none
      integer nrmax,ir,nr
      parameter (nrmax=2000)
      real nltz(nrmax),z(nrmax),sel(nrmax),selz,h,z_hub
      common/selfun_ran/ nltz,z,sel,nr
c     Locate the closest redshift
      call locate(z,nr,z_hub,ir)  !numerical recipes routine
      ir=max(ir,1)
c     Interpolate
      h=(z_hub-z(ir))/(z(ir+1)-z(ir))
      selz=sel(ir+1)*h+sel(ir)*(1.0-h)
      return
      end
c --------------------------------------------------
c  Read the Tabulated Selection Function File 
c  into common /selfun/
c
      subroutine read_sel(selfile,zlimit,vlim)
      implicit none
      character selfile*(*)
      integer nrmax,nr
      parameter (nrmax=2000)
      real sel(nrmax),r,z(nrmax),nltz(nrmax),zlimit,sel_zlimit
      logical vlim
      common/selfun/ nltz,z,sel,nr
      common /selfun_vlim/ sel_zlimit
      open(10,file=selfile,status='old',form='formatted')
      write(0,*) 'loading /selfun/ from file',selfile
      nr=0
      read(10,*)
      read(10,*)
c Magnitude limited: N(<r)
      if (.not.vlim) then
         do while ((nr.lt.nrmax).and.(z(max(nr,1)).le.zlimit))
            nr=nr+1
            read(10,*,err=12) z(nr),r,nltz(nr),sel(nr) 
         end do
c volume limited: Vol(<r)
      else
         do while ((nr.lt.nrmax).and.(z(max(nr,1)).le.zlimit))
            nr=nr+1
            read(10,*,err=12) z(nr),r,nltz(nr),sel(nr) 
            sel_zlimit=sel(nr)
            nltz(nr)=r**3
            sel(nr)=0.0
         end do
c         write(0,*) 'sel_zlimit=',sel_zlimit
      end if

      close(10)
      return

 12   continue
      stop 'z_limit is higher than tabulated z!'
      end

c----------------------------------------------------------------------------
      subroutine duplicate_common_selfun

      implicit none
      integer nrmax
      parameter (nrmax=2000)
      integer ir,nr,nr_ran
      real nltz(nrmax),z(nrmax),sel(nrmax)
      real nltz_ran(nrmax),z_ran(nrmax),sel_ran(nrmax)
      common/selfun/ nltz,z,sel,nr
      common/selfun_ran/ nltz_ran,z_ran,sel_ran,nr_ran

      write(0,*) 'Identification of common/selfun/ & common/selfun_ran/'
      do ir=1,nr
         nltz_ran(ir)=nltz(ir)
         z_ran(ir)=z(ir)
         sel_ran(ir)=sel(ir)
      enddo
      nr_ran=nr

      return
      end

c-----------------------------------------------------------------------------
c
c     GEOMETRIES.f (essentials from...)
c
c-----------------------------------------------------------------------------
c geometries.f : subroutines which define the geometry of the
c                sdss 2df_sgp 2df_ngp survey regions.
c
c USAGE : each subroutine takes the arguments (in,xo,yo,zo,r)
c
c  INPUT:
c    xo,yo,zo  (REAL*4) Cartesian galaxy position
c    r         (REAL*4) related distance r=sqrt(xo**2+yo**2+zo**2).  
c
c OUTPUT:
c    in   (LOGICAL)    set to .true.  if the galaxy is in the survey
c                      set to .false. if the galaxy is outside the survey
c
c---------------------------------------------------------------
c  2dF SGP strip
c
c subroutine that specifies the geometrical constraints of the
c survey, in = .true. is returned if the coordinate is in the survey
c The coordinates are also transformed to the specified basis and
c returned in xo,yo,zo, if in the survey.
c
c  2dF survey bounds from Steve Maddox 27 May 1997
c
c 
c 781,782,783,784,785,786,787,788,789,790,791,792,793,794,795,
c 853,854,855,856,857,858,859,860,861,862,863,864,865,866,867
c
c and 
c
c 349,350,351,352,353,354,355,356,357,404,405,406,407,408,
c 409,410,411,412,413,414,415,416,417,418,466,467,468,469,470,471,
c 472,473,474,475,476,477,478,479,480,481,532,533,534,535,536,537
c
c
c If I have done my sums right this should correspond to the following
c boundaries defined in 1950 coords. 
c
c In the NGP   dec between +2 30 0 and -7 30 0  
c             ra  between  9 50 0 to 14 50 0 
c
c In the SGP the three dec strips have different ra limits
c
c dec -22 30 0 to -27 30 0   ra between 21 48 00 to 3 24 00 
c dec -27 30 0 to -32 30 0   ra between 21 39 30 to 3 43 30 
c dec -32 30 0 to -37 30 0   ra between 21 49 00 to 3 29 00 
c
c
      subroutine geometry_2df_sgp(in,xo,yo,zo,r)
****************************variables*********************************
      implicit none
      integer ifirst
      real r,xo,yo,zo,ra,PI
      real dec1,dec2,dec3,dec4,ra1a,ra2a,ra1b,ra2b,ra1c,ra2c,off,pimoff
      parameter(PI=3.1415926)
      logical in
      real sin1,sin2,sin3,sin4
      save ifirst,sin1,sin2,sin3,sin4,ra1a,ra2a,ra1b,ra2b,ra1c,ra2c,off,
     &     pimoff
      data ifirst/1/
**********************************************************************
c   Set angular boundary limits and evaluate 
c   sin(dec) values on the first call only
      if (ifirst.eq.1) then
         ifirst=0
c
         ra1a=21.8      * PI/12.0
         ra2a= 3.4      * PI/12.0
         ra1b=21.658333 * PI/12.0
         ra2b= 3.725    * PI/12.0
         ra1c=21.816667 * PI/12.0
         ra2c= 3.48333  * PI/12.0
         ra1a=mod(ra1a+pi,2*pi)-pi !switch to interval [-pi,pi]
         ra2a=mod(ra2a+pi,2*pi)-pi 
         ra1b=mod(ra1b+pi,2*pi)-pi !switch to interval [-pi,pi]
         ra2b=mod(ra2b+pi,2*pi)-pi 
         ra1c=mod(ra1c+pi,2*pi)-pi !switch to interval [-pi,pi]
         ra2c=mod(ra2c+pi,2*pi)-pi 
         if (ra1a.gt.ra2a) then
            off=2.0*pi
         else
            off=0.0
         end if
         pimoff=pi-off
c
         dec1 = -37.5 *PI/180.0
         dec2 = -32.5 *PI/180.0
         dec3 = -27.5 *PI/180.0
         dec4 = -22.5 *PI/180.0
         sin1=sin(dec1)
         sin2=sin(dec2)
         sin3=sin(dec3)
         sin4=sin(dec4)
      end if
c
c  Check galaxy has a declination that overlaps with the region
       if ((zo-sin1*r)*(sin4*r-zo).le.0.0) then
         in=.false.
       else
         in=.true.
       end if
c 
c  If so check that it has an RA within the range of the corresponding
c  declination strip
      if (in) then
         in=.false.
         ra=atan2(yo,xo)
         if      ((zo-sin1*r)*(sin2*r-zo).ge.0.0) then
            if((ra-ra1a)*(ra2a-ra)*pimoff.ge.0.0) in=.true.
         else if ((zo-sin2*r)*(sin3*r-zo).ge.0.0) then
            if((ra-ra1b)*(ra2b-ra)*pimoff.ge.0.0) in=.true.
         else if ((zo-sin3*r)*(sin4*r-zo).ge.0.0) then
            if((ra-ra1c)*(ra2c-ra)*pimoff.ge.0.0) in=.true.         
         end if
      end if
c
      return
      end 

c---------------------------------------------------------------------------
c     Gives the general boundaries of the sgp region, in order not to compute 
c     too many randoms uselessly. Slightly conservative values...
c     Note however that theta and phi are between 0 & PI and 0 & 2PI.
      subroutine geometry_2df_sgp2(dcos,costhemin,dphi,phimin)

      implicit none

      real dcos,costhemin,dphi,phimin
      real PI
      
      PI=4.*atan(1.)
      dcos=cos(110.*PI/180.)-cos(130.*PI/180.)
      costhemin=cos(130.*PI/180.)
      dphi=((360.-320.)+60.)*PI/180.
      phimin=-40.*PI/180.

      return
      end

c----------------------------------------------------------------------------
c  2dF NGP strip
c
c subroutine that specifies the geometrical constraints of the
c survey, in = .true. is returned if the coordinate is in the survey
c The coordinates are also transformed to the specified basis and
c returned in xo,yo,zo, if in the survey.
c
c  2dF survey bounds from Steve Maddox 27 May 1997
c
c 
c 781,782,783,784,785,786,787,788,789,790,791,792,793,794,795,
c 853,854,855,856,857,858,859,860,861,862,863,864,865,866,867
c
c and 
c
c 349,350,351,352,353,354,355,356,357,404,405,406,407,408,
c 409,410,411,412,413,414,415,416,417,418,466,467,468,469,470,471,
c 472,473,474,475,476,477,478,479,480,481,532,533,534,535,536,537
c
c
c If I have done my sums right this should correspond to the following
c boundaries defined in 1950 coords. 
c
c In the NGP   dec between +2 30 0 and -7 30 0  
c              ra  between  9 50 0 to 14 50 0 
c
c In the SGP the three dec strips have different ra limits
c
c dec -22 30 0 to -27 30 0   ra between 21 48 00 to 3 24 00 
c dec -27 30 0 to -32 30 0   ra between 21 39 30 to 3 43 30 
c dec -32 30 0 to -37 30 0   ra between 21 49 00 to 3 29 00 
c
c
      subroutine geometry_2df_ngp(in,xo,yo,zo,r)
****************************variables*********************************
      implicit none
      integer ifirst
      real r,xo,yo,zo,ra,PI
      real dec1,dec2,ra1,ra2,off
      parameter(PI=3.1415926)
      logical in
      real sin1,sin2,pimoff
      save ifirst,sin1,sin2,ra1,ra2,off,pimoff
      data ifirst/1/
**********************************************************************
c   Set angular boundary limits and evaluate 
c   sin(dec) values on the first call only
      if (ifirst.eq.1) then
         ra1= 9.83333 *     PI/12.0
         ra2=14.83333 *     PI/12.0
         ra1=mod(ra1+pi,2*pi)-pi !switch to interval [-pi,pi]
         ra2=mod(ra2+pi,2*pi)-pi 
         if (ra1.gt.ra2) then
            off=2.0*pi
         else
            off=0.0
         end if
         pimoff=pi-off
c
         dec1 = -7.5 *PI/180.0
         dec2 =  2.5 *PI/180.0
         ifirst=0
         sin1=sin(dec1)
         sin2=sin(dec2)
c        write(*,*) 'sin1=',sin1
c        write(*,*) 'sin2=',sin2
      end if
c
c  Check galaxy has a declination that overlaps with the region
      in=.false.
       if ((zo-sin1*r)*(sin2*r-zo).ge.0.0) then
c        if in dec bounds check RA bounds
          ra=atan2(yo,xo)
         if((ra-ra1)*(ra2-ra)*pimoff.ge.0.0) in=.true.
       end if
c
      return
      end 

c---------------------------------------------------------------------------
      subroutine geometry_2df_ngp2(dcos,costhemin,dphi,phimin)
c     Gives the general boundaries of the ngp region, in order not to compute 
c     too many randoms uselessly. Slightly conservative values...
c     Note however that theta and phi are between 0 & PI and 0 & 2PI.

      implicit none

      real dcos,costhemin,dphi,phimin
      real PI
      
      PI=4.*atan(1.)
      dcos=cos(85.*PI/180.)-cos(100.*PI/180.)
      costhemin=cos(100.*PI/180.)
      dphi=(225.-140)*PI/180.
      phimin=140.*PI/180.

      return
      end

c--------------------------------------------------------------------------
c     2dF random fields:
c
c subroutine that specifies the geometrical constraints of the
c survey, in = .true. is returned if the coordinate is in the survey
c
c N.B.: set nbf to 0 if one doesn't know in which field to look. Otherwise
c       returns true or false for galaxy belonging to field number nbf.
c

      subroutine geometry_2df_ran(in,xo,yo,zo,r,nbf)

      implicit none

      logical in
      integer nbf
      real xo,yo,zo,r

      integer nmax
      real theta_min
      parameter (nmax=100,theta_min=1.014)
      integer i,ifirst,nb_field
      real costheta,costheta_r,costheta_min,xc(nmax),yc(nmax),zc(nmax)
     &,    ra(nmax),dec(nmax)
      data ifirst/1/
      save nb_field,xc,yc,zc,ra,dec,ifirst,costheta_min


      if (ifirst.eq.1) then
         ifirst=0
         call ran2df_cen(nb_field,xc,yc,zc,ra,dec)
         costheta_min=cos(theta_min*atan(1.)/45.)
      endif

      in=.false.      
      costheta_r=costheta_min*r

      if (nbf.ne.0) then
         costheta=(xo*xc(nbf)+yo*yc(nbf)+zo*zc(nbf))
         if (costheta.gt.costheta_r) in=.true.
      else
         i=0
         costheta_r=costheta_min*r
         do while ((.not.in).and.(i.lt.nb_field))
            i=i+1
            costheta=(xo*xc(i)+yo*yc(i)+zo*zc(i))
            if (costheta.gt.costheta_r) in=.true.
         enddo
      endif

      return
      end

c--------------------------------------------------------------------------
c     Gives the general boundaries of the sgp region, in order not to compute 
c     too many randoms uselessly.
      subroutine geometry_2df_ran2(dcos,costhemin,dphi,phimin,nbf)

      implicit none

      integer nbf
      real dcos,costhemin,dphi,phimin
      real PI

      PI=4.*atan(1.)
      costhemin=cos(165.*PI/180.)
      dcos=cos(80.*PI/180.)-costhemin
      dphi=((360.-285.)+90.)*PI/180.
      phimin=-75.*PI/180.
      nbf=0

      return
      end

c-----------------------------------------------------------------------------
c     Select randomly a random field in the 2dF survey and give the boundaries
c     
      subroutine geometry_2df_ran3(iseed,dcos,costhemin,dphi,phimin,nbf)

      implicit none

      integer iseed,nbf
      real dcos,costhemin,dphi,phimin

      integer nmax
      real theta_min,PI_2
      parameter (PI_2=1.570796327)
      parameter (nmax=100,theta_min=1.014*PI_2/90.)
      integer ifirst,nb_field
      real xc(nmax),yc(nmax),zc(nmax),ra(nmax),dec(nmax)
      real dcos_vec(nmax),costhemin_vec(nmax),dphi_vec(nmax),phimin_vec(nmax)
      real costheta,sintheta,angle,ramin,ramax,rnbf,ran3

      data ifirst/1/
      save rnbf,dcos_vec,costhemin_vec,dphi_vec,phimin_vec
     &,    nb_field,xc,yc,zc,ra,dec,ifirst,costheta


      if (ifirst.eq.1) then
         ifirst=0
         call ran2df_cen(nb_field,xc,yc,zc,ra,dec)
         rnbf=real(nb_field)
         costheta=cos(theta_min)
         sintheta=sin(theta_min)
         do nbf=1,nb_field

            costhemin_vec(nbf)=sin(dec(nbf))*costheta-cos(dec(nbf))*sintheta
            dcos_vec(nbf)=2.*cos(dec(nbf))*sintheta
            angle=theta_min/cos(dec(nbf))
            ramax=ra(nbf)+angle
            ramin=ra(nbf)-angle

            if (ramax.gt.ramin) then
               dphi_vec(nbf)=ramax-ramin
            else
               dphi_vec(nbf)=(4.*PI_2+ramax)-ramin
            endif
            phimin_vec(nbf)=ramin
         enddo
      endif

c     Select a field randomly
      nbf=min(int(ran3(iseed)*rnbf+1.),nb_field)

      costhemin=costhemin_vec(nbf)
      dcos=dcos_vec(nbf)

      phimin=phimin_vec(nbf)
      dphi=dphi_vec(nbf)

      return
      end

c-----------------------------------------------------------------------------
c     This subroutine reads the 2dF random field centres from the file name.
      subroutine ran2df_cen(nb_field,xc,yc,zc,ra,dec)

      implicit none

      integer nb_field
      real xc(*),yc(*),zc(*),ra(*),dec(*)

      integer istat,i,ix,iy,j,k
      real x,y
      real const1,const2
      character name*80

c     Set up some constants
      const1= 4.*atan(1.)/(12.*3600.)
      const2= 4.*atan(1.)/180.
      name='sub/rancentres.txt'
c      name='/gal/dsk18/irpn/2df/data/rancentres_apm.txt' 
c     contains one extra 2dF field which actually has astrometry problems in..

c      write(*,*) 'file: ', name
      istat=0
      i=0
      open(unit=13,file=name,status='old',form='formatted')
      do while (istat.eq.0)
         read(13,130,iostat=istat) ix,iy,x,j,k,y
 130     format(2X,I2,1X,I2,1X,f5.2,2X,I3,1X,I2,1X,f4.1)
         if (istat.eq.0) then
            i=i+1
            ra(i)=(real(ix)*3600.+real(iy)*60.+x)*const1
            dec(i)=(real(j)+real(sign(1,j))*(real(k)*60.+y)/3600.)*const2
            xc(i)=cos(dec(i))*cos(ra(i))
            yc(i)=cos(dec(i))*sin(ra(i))
            zc(i)=sin(dec(i))
         endif
      enddo
      close(13)
      nb_field=i
c      write(*,*) 'Number of random fields:',nb_field

      return
      end

c-----------------------------------------------------------------------------
