      subroutine check_pix_ran(NP_ran,NPX_ran)

      implicit none

      include 'pix_ran.inc'
      integer NP_ran,NPX_ran


      if ((NP_ran.ne.NP).or.(NPX_ran.ne.NPX)) then
         write(0,*) 'pix_ran.inc values are not correct:'
         write(0,*) 'NP_ran=',NP_ran,' NPX_ran=',NPX_ran
         write(0,*) 'NP    =',NP    ,' NPX    =',NPX
         stop 'check_pix_ran in ranmask.f!'
      endif

      return
      end

c----------------------------------------------------------------------------
      subroutine readranmask(namemask,NPX,NPY,nbf,rpix)

c     This subroutine reads the mask stored in the file name_mask.
c     N.B.: This subroutine contains the same stuff as mask_2df, but has the 
c     advantage of giving back also sgn!

      implicit none

      integer nbf,NPX,NPY
      real rpix(nbf,NPX,NPY)
      character namemask*(*)
      
      integer i,j,k,npxx,npyy,nbff

      open(unit=14,file=namemask,status='old',form='unformatted')
      read(14) nbff,npxx,npyy
      if ((npxx.ne.NPX).or.(npyy.ne.NPY).or.(nbff.ne.nbf)) then
        write(*,*) 'Wrong dimensions for ran mask. Should have:'
     &,             npxx,npyy,nbff
        stop 'readranmask'
      endif
      do k=1,nbff
         do i=1,npxx
            read(14) (rpix(k,i,j),j=1,npyy)
         enddo
      enddo
      close(14)

      return
      end

c----------------------------------------------------------------------------
      subroutine ranmask_val(namemask,x,y,z,pixval)

      implicit none
      real x,y,z,pixval
      character namemask*(*)

      include 'pix_ran.inc'
      real theta_min
      parameter (theta_min=1.014)
      integer ifirst,nb_field,inbf,ix,iy
      real costheta,costheta_r,costheta_min,xc(NBF),yc(NBF),zc(NBF)
     &,    rpix(NBF,NPX,NPY)
      character namemask_save*180
      data ifirst/1/
      save ifirst,xc,yc,zc,costheta_min,rpix,namemask_save

      character ranfield_centre*180,ranfield_area*180
      common /ranfieldfile/ ranfield_centre,ranfield_area


c     On the first call, reads the file containing the random fields
      if ((ifirst.eq.1).or.(namemask.ne.namemask_save)) then
         if (ifirst.eq.1) 
     &        call ran2df_cen_name(ranfield_centre,nb_field,xc,yc,zc)

         ifirst=0.         
         namemask_save=namemask
         call readranmask(namemask,NPX,NPY,NBF,rpix)

         if (nb_field.ne.NBF) stop 'ranmask_val: wrong NBF or nb_field!'
         costheta_min=cos(theta_min*atan(1.)/45.)
      endif

      call get_inbf_ix_iy(NP,NPX,NPY,NBF,xc,yc,zc,costheta_min
     &,                   x,y,z,inbf,ix,iy)

      if (inbf.ge.1) then
         pixval=rpix(inbf,ix,iy)
      else
         pixval=-2. ! Default value
      endif

      return
      end

c-----------------------------------------------------------------------------
      subroutine get_inbf_ix_iy(NP,NPX,NPY,nbf,xc,yc,zc,costheta_min
     &,          x,y,z,inbf,ix,iy)

      implicit none

      integer NP,NPX,NPY,nbf,inbf,ix,iy
      real xc(*),yc(*),zc(*),costheta_min,x,y,z

      integer i,io,ixc_min,iyc_min,sgn
      real rx,ry,costheta_r,costheta,ra,dec


      i=0                       ! initializing value
      inbf=0                    ! default value

      costheta_r=costheta_min*sqrt(x**2+y**2+z**2)
      do while ((inbf.eq.0).and.(i.lt.nbf))
         i=i+1
         costheta=(x*xc(i)+y*yc(i)+z*zc(i))
         if (costheta.gt.costheta_r) then ! We use here the fact the random
            inbf=i                        ! fields doesn't overlapp with 
         endif                            ! each other...
      enddo

      if (inbf.ne.0) then
         call initialisation(1) ! value always 1, as random fields are in SGP
         call xyz_radec(x,y,z,ra,dec)
         call test_hole(ra,dec,io) ! hole (io=0) or not (io=1)
c                 Good position <==> io=1 && inbf>=1
         if (io.eq.0) then 
            inbf=0              ! default value
         else
            call eq_2dfrx(NP,xc(i),yc(i),zc(i),rx,ry,sgn)
            call bound_map_ran(NPX,NPY,NP,sgn,rx,ry,ixc_min,iyc_min)

            call eq_2dfrx(NP,x,y,z,rx,ry,sgn)
            call rx_ix_map(NP,sgn,rx,ry,ix,iy)
      
            ix=min(max(1,ix-ixc_min),NPX)
            iy=min(max(1,iy-iyc_min),NPY)

         endif
      endif

      return
      end

c-----------------------------------------------------------------------------
c     This subroutine reads the 2dF random field centres from the file name.
      subroutine ran2df_cen_name(name,nb_field,xc,yc,zc)

      implicit none

      integer nb_field
      real xc(*),yc(*),zc(*)
      character name*(*)

      integer istat,i,ix,iy,j,k
      real x,y
      real*8 ra,dec,const1,const2


c     Set up some constants
      const1= 4.*atan(dble(1.))/(12.*3600.)
      const2= 4.*atan(dble(1.))/180.

c      write(*,*) 'file: ', name
      istat=0
      i=0
      open(unit=13,file=name,status='old',form='formatted')
      do while (istat.eq.0)
         read(13,130,iostat=istat) ix,iy,x,j,k,y
 130     format(2X,I2,1X,I2,1X,f5.2,2X,I3,1X,I2,1X,f4.1)
         if (istat.eq.0) then
            i=i+1
            ra=(dble(ix)*3600.+dble(iy)*60.+dble(x))*const1
            dec=(dble(j)+dble(sign(1,j))*(dble(k)*60.+dble(y))/3600.)*const2
            xc(i)=cos(dec)*cos(ra)
            yc(i)=cos(dec)*sin(ra)
            zc(i)=sin(dec)
         endif
      enddo
      close(13)
      nb_field=i
c      write(*,*) 'Number of random fields:',nb_field

      return
      end

c------------------------------------------------------------------------
      subroutine read_fields_xyz(name,region,nb_field,ifield,xc,yc,zc)

      implicit none

      integer nb_field,ifield(*)
      real xc(*),yc(*),zc(*)
      character region*3,name*(*)

      integer istat,ix,iy,i,j,k,jj,ichar
      real x,y,z,z1
      real*8 ra,dec,const1,const2
      character char*1

      const1= atan(dble(1.))/dble(10800.) ! 4/(12*3600)
      const2= atan(dble(1.))/dble(45.)    ! PI 


      istat=0
      i=0
      open(unit=10,file=name,status='old',form='formatted')

      if (region.eq.'ran') then
         do while (istat.eq.0)
            read(10,100,iostat=istat) ix,iy,x,char,j,k,y,jj
 100        format(2X,I2,1X,I2,1X,f5.2,2X,a1,I2,1X,I2,1X,f4.1,2x,I3)
            if (istat.eq.0) then
               i=i+1
               ichar=+1
               if (char.eq.'-') ichar=-1
               ra=(dble(ix)*dble(3600.)+dble(iy)*dble(60.)+dble(x))*const1
               dec=(dble(j*ichar)+dble(ichar)*(dble(k)*dble(60.)+dble(y))
     &                       / dble(3600.))*const2
               ifield(i)=jj
               xc(i)=real(cos(dec)*cos(ra))
               yc(i)=real(cos(dec)*sin(ra))
               zc(i)=real(sin(dec))
            endif
         enddo
      else
         do while (istat.eq.0)
            read(10,101,iostat=istat) z,z1,ix,iy,x,j,k,y
 101        format(6X,f8.4,6X,f8.4,21X,I2,1X,I2,1X,f5.2,2X,I3,1X,I2,2X,f3.1)
            if (istat.eq.0) then
               i=i+1
               ra=(dble(ix)*dble(3600.)+dble(iy)*dble(60.)+dble(x))*const1
               dec=(dble(j)+dble(sign(1.,z1))*(dble(k)*dble(60.)+dble(y))
     &                       / dble(3600.))*const2
               ifield(i)=i
               xc(i)=real(cos(dec)*cos(ra))
               yc(i)=real(cos(dec)*sin(ra))
               zc(i)=real(sin(dec))
            endif
         enddo
      endif

      close(10)
      nb_field=i 
        
      return
      end

c-----------------------------------------------------------------------------
      subroutine ranmask_val2(maglimmask,mumaskfile,x,y,z,magval,muval)

      implicit none
      real x,y,z,magval,muval
      character maglimmask*(*),mumaskfile*(*)


      include 'pix_ran.inc'
      real theta_min
      parameter (theta_min=1.014)
      integer ifirst,nb_field,inbf,ix,iy
      real costheta,costheta_r,costheta_min,xc(NBF),yc(NBF),zc(NBF)
     &,    magpix(NBF,NPX,NPY),mupix(NBF,NPX,NPY)
      character maglimmask_save*180,mumaskfile_save*180
      data ifirst/1/
      save ifirst,xc,yc,zc,costheta_min,magpix,mupix,maglimmask_save
     &,    mumaskfile_save

      character ranfield_centre*180,ranfield_area*180
      common /ranfieldfile/ ranfield_centre,ranfield_area


c     On the first call, reads the file containing the random fields
      if ( (ifirst.eq.1).or.(maglimmask.ne.maglimmask_save).or.
     &     (mumaskfile.ne.mumaskfile_save) ) then

         if (ifirst.eq.1) 
     &        call ran2df_cen_name(ranfield_centre,nb_field,xc,yc,zc)
         
         if ((ifirst.eq.1).or.(maglimmask.ne.maglimmask_save)) then
            maglimmask_save=maglimmask
            call readranmask(maglimmask,NPX,NPY,NBF,magpix)
         endif

         if ((ifirst.eq.1).or.(mumaskfile.ne.mumaskfile_save)) then
            mumaskfile_save=mumaskfile
            call readranmask(mumaskfile,NPX,NPY,NBF,mupix)
         endif

         if (nb_field.ne.NBF)stop 'ranmask_val2: wrong NBF or nb_field!'
         costheta_min=cos(theta_min*atan(1.)/45.)
         ifirst=0.

      endif

      call get_inbf_ix_iy(NP,NPX,NPY,nbf,xc,yc,zc,costheta_min
     &,                   x,y,z,inbf,ix,iy)

      if (inbf.ge.1) then
         magval=magpix(inbf,ix,iy)
         muval=mupix(inbf,ix,iy)
      else
         muval= -2. ! Default value
         magval=-2. ! Default value
      endif

      return
      end


c-----------------------------------------------------------------------------
      subroutine ranmask_val3(maglimmask,mumaskfile,maskfile,x,y,z
     &,          magval,muval,compval)

      implicit none
      real x,y,z,magval,muval,compval
      character maglimmask*(*),mumaskfile*(*),maskfile*(*)

      include 'pix_ran.inc'
      real theta_min
      parameter (theta_min=1.014)
      integer ifirst,nb_field,inbf,ix,iy
      real costheta,costheta_r,costheta_min,xc(NBF),yc(NBF),zc(NBF)
     &,    comp(NBF),magpix(NBF,NPX,NPY),mupix(NBF,NPX,NPY)
      character*180 maglimmask_save,mumaskfile_save,maskfile_save
      data ifirst/1/
      save ifirst,xc,yc,zc,costheta_min,magpix,mupix,maglimmask_save
     &,    mumaskfile_save,maskfile_save,comp


c     On the first call, reads the file containing the random fields
      if ((ifirst.eq.1).or.(maglimmask.ne.maglimmask_save).or.
     &    (mumaskfile.ne.mumaskfile_save).or.(maskfile.ne.maskfile_save)) then

         if ((ifirst.eq.1).or.(maskfile.ne.maskfile_save)) then
            maskfile_save=maskfile
            call ran2df_cen_used(nb_field,xc,yc,zc,comp,maskfile)
            write(0,*) 'Read: ',maskfile
         endif
         
         if ((ifirst.eq.1).or.(maglimmask.ne.maglimmask_save)) then
            maglimmask_save=maglimmask
            call readranmask(maglimmask,NPX,NPY,NBF,magpix)
            write(0,*) 'Read: ',maglimmask
         endif

         if ((ifirst.eq.1).or.(mumaskfile.ne.mumaskfile_save)) then
            mumaskfile_save=mumaskfile
            call readranmask(mumaskfile,NPX,NPY,NBF,mupix)
            write(0,*) 'Read: ',mumaskfile
         endif

         if (nb_field.ne.NBF)stop 'ranmask_val3: wrong NBF or nb_field!'
         costheta_min=cos(theta_min*atan(1.)/45.)
         ifirst=0.

      endif

      call get_inbf_ix_iy(NP,NPX,NPY,nbf,xc,yc,zc,costheta_min
     &,                   x,y,z,inbf,ix,iy)

      if (inbf.ge.1) then
         magval=magpix(inbf,ix,iy)
         muval=mupix(inbf,ix,iy)
         compval=comp(inbf)
      else
         compval=0. ! Default value
         muval= -2. ! Default value
         magval=-2. ! Default value
      endif

      return
      end
      
c----------------------------------------------------------------------------
      subroutine getranmask_max(namemask,max_mask)

      implicit none
      
      real max_mask
      character namemask*(*)

      integer NPY
      parameter (NPY=1000)
      integer nbff,npxx,npyy,i,k,j
      real rpix(NPY)


      max_mask=0.
      open(unit=14,file=namemask,status='old',form='unformatted')
      read(14) nbff,npxx,npyy
      if (npyy.gt.NPY) stop 'getranmask_max: need larger NPY!'
      do k=1,nbff
         do i=1,npxx
            read(14) (rpix(j),j=1,npyy)
            do j=1,npyy
               max_mask=max(rpix(j),max_mask)
            enddo
         enddo
      enddo
      close(14)

      return
      end

c-----------------------------------------------------------------------------

