c----------------------------------------------------------------------------
c Model of APM/2dF selection function based on a Schecter function,
c including a k-correction and also taking account of the mu value.
c
c Notes:
c  Schecter Function is phistar (L/Lstar)^-alpha exp(-L/Lstar) dL/Lstar
c
c  x = L/Lstar
c
c  selz is the integral of the Schecter LF between lumiosities
c  that at the chosen redshift correspond to the bright and faint
c  magnitude limits.
c
c  selz_mu is the integral over the same range of the LF weighted
c  by the completeness 1-exp(mag-mu). However we also upweight
c  by factor equal to the inverse of this incompletness intregrated
c  over all redshift as otherwise when we multiply selz_mu by comp
c  we will double count this source of incompleteness.
c
      subroutine selfunc(selz,selz_mu,z,alpha,mstar,phistar,magmin,magmax,mu)
c*****************************************************************
      implicit none
      real r,ratio,beta,selmu_min,selmu_max,alpha_count,galpha,cf_mu
      parameter(beta=1.085736205,alpha_count=1.25542801) !0.55*log(10.0)
      real sel_max,sel_min,z,x,expint,dl,dlzc,zplus1,kpluse
      real alpha,mstar,magmax,magmin,phistar,mu,selz,selz_mu,fac
      external kpluse
c*****************************************************************
      zplus1=1.0+z
      call search_dist(r,zplus1,dl,dlzc,2)  !find corresponding dl
      fac=(dl**2)*(10.0**(0.4*kpluse(z))) 

      ratio =fac * 10.0**(10.0 -0.4*(mu-mstar))
      x = fac *  10.0**(10.0 -0.4*(magmax-mstar))
      sel_max = expint(x,-alpha) 
      selmu_max = (ratio**beta)*expint(x,-alpha+beta) 

      x = fac *  10.0**(10.0 -0.4*(magmin-mstar))
      sel_min = expint(x,-alpha) 
      selmu_min = (ratio**beta)*expint(x,-alpha+beta) 

c
c     Adopt a power law model of the counts to compute the overall
c     completeness due to mu for the specified magnitude range.
      galpha=(alpha_count/(1.0+alpha_count)) * 
     &     (exp((alpha_count+1.0)*(magmax-magmin))-1.0)
     &     /(exp(alpha_count*(magmax-magmin))-1.0)
      cf_mu= 1.0 -galpha*exp( magmin - mu )

c
c     Evaluation galaxy number density ignoring mu value and any
c     other incompleteness.
      selz = phistar*(sel_max-sel_min)
c
c     Evaluation galaxy number density taking into account the mu value 
c     but then upweighting by the overall incompleteness for this mu.
      selz_mu = (selz -phistar*(selmu_max-selmu_min))/cf_mu
      
c      if (selz_mu.lt.0.0 .and. mu.gt.19.0) then
c         write(0,*) 'selz,selz_mu',selz,selz_mu
c         write(0,*) 'z,alpha,mstar,phistar,magmin,magmax,mu'
c         write(0,*) z,alpha,mstar,phistar,magmin,magmax,mu
c         write(0,*) 'cf_mu,(selmu_max-selmu_min)*phistar,selmu_max,selmu_min'
c         write(0,*) cf_mu,(selmu_max-selmu_min)*phistar,selmu_max,selmu_min
c         write(0,*) 'ratio,fac'
c         write(0,*)  ratio,fac
c         stop 'selfunc() error.'
c      end if


      return
      end

