c Tabulae the selection function given
c Schechter function parameters, a magnitude limit and
c cosmological parameters Omega and Lambda
c and store in common block /selfun_ran/

      subroutine tabulate_sel_ran(phistar,mstar,alpha,magmax,magmin)
************************************variables********************************
      implicit none
      real PI
      integer nr,NRMAX
      parameter(PI=3.141592654,nrmax=2000)
      real phistar,mstar,alpha,magmax,magmin,sel,dv,omega_s,n_str,r,dr
     &,    zplus1,dl,dlzc,alpha0,mstar0,phistar0,magmax0,magmin0
      integer ir
      external sel
      common /lfpara/ alpha0,mstar0,phistar0,magmax0,magmin0
      real selz(nrmax),z(nrmax),nltz(nrmax)
      common/selfun_ran/ nltz,z,selz,nr

******************************************************************************
      if (alpha.ge.0) 
     &    stop 'tabulate_sel: Negative alpha is the adopted convention!'

      alpha0=alpha
      mstar0=mstar
      phistar0=phistar
      magmax0=magmax
      magmin0=magmin

      omega_s=(PI/180.0)**2  ! nominal 1 square degree

c     Save selection function to the specified file 
      nr=NRMAX
      n_str=0.0
      r=0.0
      dr=1900.0/real(nr)
      do ir=1,nr
         dv=((r+dr)**3 - r**3)/3.0 
         n_str=n_str+sel(r+0.5*dr)*dv
c        find corresponding zplus1 and dl
         call search_dist((r+0.5*dr),zplus1,dl,dlzc,1)  
c        store to common block /selfun_ran/
         z(ir)=zplus1-1.0
         selz(ir)=sel(r+0.5*dr)
         nltz(ir)=n_str*omega_s
         r=r+dr
      end do

      return
      end

c----------------------------------------------------------------------------
      subroutine get_nsqdg(nsqdg)

      implicit none

      real nsqdg

      integer NRMAX,nr
      parameter(nrmax=2000)
      real selz(nrmax),z(nrmax),nltz(nrmax)
      common/selfun_ran/ nltz,z,selz,nr      

      nsqdg=nltz(nr)  !number of objects per sq deg to magnitude limit

      return
      end

c----------------------------------------------------------------------------
c Model of APM selection function based on a schechter function
c and including a k-correction
c
c  r in Mpc/h
c
      real function sel(r)
c*****************************************************************
      implicit none
      real r
      real sel_max,sel_min,z,x,expint,dl,dlzc,zplus1,kpluse
      real alpha,mstar,magmax,magmin,phistar
      external kpluse
      common /lfpara/ alpha,mstar,phistar,magmax,magmin
c*****************************************************************
      call search_dist(r,zplus1,dl,dlzc,1)  !find corresponding zplus1 and dl
      z = zplus1-1.0
      x = (dl**2)*(10.0**(0.4*kpluse(z))) * 
     &             10.0**(10.0 -0.4*(magmax-mstar))

      sel_max = phistar*expint(x,-alpha) 
      
      x = (dl**2)*(10.0**(0.4*kpluse(z))) * 
     &             10.0**(10.0 -0.4*(magmin-mstar))

      sel_min = phistar*expint(x,-alpha) 

      sel = sel_max-sel_min

      return
      end

